/* $Id: hellosign.c,v 1.29 2016/05/13 07:31:18 stm Exp $
 *
 * PDFlib PLOP DS:
 * generate a document in memory with PDFlib and sign it with PLOP DS
 */

#include <stdio.h>

#include "pdflib.h"
#include "ploplib.h"

int
main(void)
{
    PDF *p;
    PLOP *plop;
    int font;
    volatile const char *volatile contents = NULL;
    long size;

    /* This is where input files live. Adjust as necessary. */
    const char *searchpath = "../data";

    const char *out_filename = "hellosign.pdf";

    /* Basic signature options */
    static const char sign_opts[] =
        "ltv=try "          /* use ltv=full to ensure LTV status */
#if defined(USE_PKCS11)
        "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
#else
        "engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} "
        "password={demo} "
#endif
    ;

    if ((p = PDF_new()) == (PDF *) 0)
    {
        printf("Couldn't create PDFlib object (out of memory)!\n");
        return(98);
    }

    /* ---------- Step 1: generate document in memory with PDFlib ---------- */

    PDF_TRY(p) {
	/* create a new PDF document in memory */

	PDF_begin_document(p, "", 0, "");

	PDF_set_info(p, "Creator", "hellosign.c");
	PDF_set_info(p, "Author", "Rainer Ploeckl");
	PDF_set_info(p, "Title", "Hello, world/signed (C)!");

	font = PDF_load_font(p, "Helvetica-Bold", 0, "host", "");

	PDF_begin_page_ext(p, a4_width, a4_height, "");

	PDF_setfont(p, font, 24);
	PDF_set_text_pos(p, 50, 700);
	PDF_show(p, "Hello, world/signed!");
	PDF_continue_text(p, "(says C)");
	PDF_end_page_ext(p, "");

	PDF_end_document(p, "");

	/* fetch the memory buffer containing the PDF document */
	contents = PDF_get_buffer(p, &size);
    }

    PDF_CATCH(p) {
        printf("PDFlib exception occurred in hello sample:\n");
        printf("[%d] %s: %s\n",
            PDF_get_errnum(p), PDF_get_apiname(p), PDF_get_errmsg(p));
        PDF_delete(p);
        return(98);
    }

    /* ------------------ Step 2: sign the document ----------------- */

    /* create a new PLOP object */
    if ((plop = PLOP_new()) == (PLOP *) 0)
    {
	printf("Couldn't create PLOP object (out of memory)!\n");
	return(99);
    }

    PLOP_TRY(plop) {
	int doc;

	char optlist[1024];

	sprintf(optlist, "searchpath={%s}", searchpath);
	PLOP_set_option(plop, optlist);

	/* Create a virtual PVF file from the document data in memory */
	PLOP_create_pvf(plop, "/pvf/hello.pdf", 0,
		(const char *) contents, size, "");

	/* open input document from PVF file */
	if ((doc = PLOP_open_document(plop, "/pvf/hello.pdf", 0, "")) == -1)
	{
	    printf("Error: %s\n", PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    PDF_delete(p);
	    return(99);
	}

	if (PLOP_prepare_signature(plop, sign_opts) == -1)
	{
	    printf("Error: %s: %s\n",
		    PLOP_get_apiname(plop), PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    return(2);
	}

	/* sign the output file */
	sprintf(optlist, "input=%d", doc);
	if (PLOP_create_document(plop, out_filename, 0, optlist) == -1)
	{
	    printf("Error: %s\n", PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    PDF_delete(p);
	    return(99);
	}

	/* close the input document */
        PLOP_close_document(plop, doc, "");

	/* delete PVF file containing input document */
	if (PLOP_delete_pvf(plop, "/pvf/hello.pdf", 0) == -1)
	{
	    printf("Error: %s\n", PLOP_get_errmsg(plop));
	    PLOP_delete(plop);
	    PDF_delete(p);
	    return(99);
	}
    }

    PLOP_CATCH(plop) {
	printf("PLOP exception occurred in hellosign:\n");
	printf("[%d] %s: %s\n", PLOP_get_errnum(plop),
		PLOP_get_apiname(plop), PLOP_get_errmsg(plop));
	PLOP_delete(plop);
	return(99);
    }

    /* delete the PLOP object */
    PLOP_delete(plop);

    /* delete the PDFlib object */
    PDF_delete(p);

    return 0;
}
